#!/usr/bin/env python

from parallel import start, Exchange
from Dict import Indexer, Searcher, Parser

def apply_parser(channel, filename):
    print "Parsing", filename
    parser = Parser(filename)
    parser.send_entries(channel)

def get_searcher(filenames, window_size):
    master_index = Indexer()

    # Start indexing.

    for i in range(0, len(filenames), window_size):
        channels = []
        for filename in filenames[i:i + window_size]:
            channels.append(start(apply_parser, filename))

        # Start listening for responses.

        exchange = Exchange(channels)
        while exchange.active():
            print "Waiting for %d channels..." % exchange.active()
            for channel in exchange.ready():
                entry = channel.receive()
                master_index.add_entry(entry)

        # Tidy up as we go.

        for channel in channels:
            channel.close()

    # Provide a search interface.

    return Searcher(master_index.get_index())

if __name__ == "__main__":
    import sys, os

    # Get the parameters.

    window_size, directory = int(sys.argv[1]), sys.argv[2]

    # Build a list of filenames.

    filenames = [os.path.join(directory, filename) for filename in os.listdir(directory)]
    filenames = [filename for filename in filenames if os.path.isfile(filename)]

    # Get a searcher using an index built in parallel.

    searcher = get_searcher(filenames, window_size)

    # Wait for all created processes to terminate.

    try:
        while 1:
            os.wait()
    except OSError:
        pass

    # Present a user interface.

    while 1:
        print "Pattern:",
        pattern = raw_input()
        print searcher.find(pattern)

# vim: tabstop=4 expandtab shiftwidth=4
